## 伪类选择器（Pseudo-classes selectors）
> CSS伪类（pseudo-class）是加在选择器后面的用来指定`元素状态`的关键字。

### 常用伪类列表
- `:link`
    > `:link`伪类选择器是用来选中元素当中的链接。它将会选中`所有尚未访问`的链接，包括那些已经给定了其他伪类选择器的链接（例如`:hover`选择器，`:active`选择器，:`visited`选择器）。为了可以正确地渲染链接元素的样式，`:link`伪类选择器应当放在其他伪类选择器的前面，并且遵循`LVHA的先后顺序`（参考[样式优先级](/CSS/1样式优先级.md)），即：`:link` — `:visited` — `:hover` — `:active`。`:focus`伪类选择器常伴随在`:hover`伪类选择器左右，需要根据你想要实现的效果确定它们的顺序。
- `:visited`
    > 只向已被访问的链接添加样式
- `:active`
    > 匹配被用户激活的元素。它让页面能在浏览器监测到激活时给出反馈。当用鼠标交互时，它代表的是用户按下按键和松开按键之间的时间。 `:active`伪类通常用来匹配`tab`键交互。通常用于但并不限于`<a>` 和 `<button>`HTML元素。
- `:hover`
    > 适用于用户使用指示设备虚指一个元素（没有激活它）的情况。这个样式会被任何与链接相关的伪类重写，像`:link`, `:visited`, 和 `:active`等。
- `:focus`
    > 在一个元素成为焦点时生效，用户可以通过键盘或鼠标激活焦点（例如：一次表单输入）。
- `:first-child`
    > 代表了一组兄弟元素中的第一个元素。在`level3`实现中，被匹配的元素需要具有一个父级元素，而在`level4`实现中则不需要。与`:first-of-type`类似，但不同之处在于`:first-child`用来选择父元素的第一个子元素的元素E的同时，元素E必须是父元素的第一个子元素（且不是后代元素）示例[`:first-child`与`:first-of-type`的差异](#example1)
- `:nth-child`
    > 匹配文档树中所有下标在集合`{ an + b; n = 0, 1, 2, ...} `（`a` 和 `b` 都必须为整数，并且元素的第一个子元素的下标为 1）中的子元素。
- `:nth-last-child`
    > 基本上和 `:nth-child` 一样，只是它从结尾处反序计数，而不是从开头处。
- `:nth-of-type`
- `:first-of-type`
- `:last-of-type`
- `:empty`
    > 匹配没有子元素的元素。这里说的子元素，只计算元素结点及文本（包括`空格`），注释、运行指令不考虑在内。
- `:target`
    > 带有片段标识符的URI链接到文档中某一个元素（即目标元素）时的状态
- `:checked`
    > 匹配任何处于选中状态的`radio`(`<input type="radio">`), `checkbox` (`<input type="checkbox">`) 或 `select` 元素中的`option`。
- `:enabled`
    > 匹配任何启用的（`enabled`）元素。
- `:disabled`
    > 匹配任何被禁用的（`disabled`）元素。


### 完整伪类列表
参见[CSS3 Selectors working spec](http://www.w3.org/TR/selectors/#selectors)

## 基于关系的选择器

### 常见的基于关系的选择器
- `A E`	后代选择器，元素A的任一后代元素E (后代节点指A的子节点、子节点的子节点，以此类推)
- `A > E`	子选择器，元素A的任一子元素E(也就是直系后代)
- `E:first-child`	任一的元素E（它是其父母结点的第一个子节点）
- `B + E`	相邻兄弟选择器，该选择器定位与元素B具有相同父元素且在HTML DOM中紧邻B的元素E
- `B ~ E`	一般兄弟选择器，该选择器定位与元素B具有相同父元素且在HTML DOM中位于B之后的所有元素E

### 示例

<h3 id='example1'><code>:first-child</code>与<code>:first-of-type</code>的差异<h3>

> 参考自[伪类选择器之:first-child和:first-of-type](http://blog.leanote.com/post/wechat-aaf3/a0812eceab0d)

渲染下面的HTML：
``` html
    <div class="wrapper">
        <div>lalala</div>
        <p>passage1</p>
        <p>passage2</p>
        <div>kakaka</div>
    </div>
```
``` css
    /*:first-child 渲染无效*/
    .wrapper>p:first-child{
        background:red;
    }
    /*:first-of-type 渲染有效*/
    .wrapper>p:first-of-type{
        background:red;
    }
```