> 在样式中可以定义文本内容和图片内容，就能把文本内容及图片添加到文档。样式表中指定的内容`不会成为`DOM的一部分。

## 文本内容
- `::before`会为当前元素创建一个子元素作为伪元素。常通过`content`属性来为一个元素添加修饰性的内容。 此元素默认为行内元素。
- `::after`用来创建一个伪元素，做为已选中元素的最后一个子元素。通常会配合`content`属性来为该元素添加装饰内容。这个虚拟元素默认是行内元素。

## 图片内容
> 可以通过将content 属性值设置为某个图片的URL，可以将图片插到元素的前面或后面。

## 示例
### `::before`
渲染下面的HTML：
``` html
    <q>一些引用</q>, 他说, <q>比没有好。</q>
```
``` css
    q::before { 
        content: "“";
        color: blue;
    }
    q::after { 
        content: "”";
        color: red;
    }
```
输出如图：![::before](/resource/image/2017-10-19_205834.png)
### `::after`
渲染下面的HTML：
``` html
<p>这是上面代码的实现<br />
  我们有一些 <span data-descr="collection of words and punctuation">文字</span> 有一些
  <span data-descr="small popups which also hide again">提示</span>。<br />
  把鼠标放上去<span data-descr="not to be taken literally">看看</span>.
</p>
```
``` css
span[data-descr] {
  position: relative;
  text-decoration: underline;
  color: #00F;
  cursor: help;
}

span[data-descr]:hover::after {
  content: attr(data-descr);
  position: absolute;
  left: 0;
  top: 24px;
  min-width: 200px;
  border: 1px #aaaaaa solid;
  border-radius: 10px;
  background-color: #ffffcc;
  padding: 12px;
  color: #000000;
  font-size: 14px;
  z-index: 1;
}
```
输出如图：![::before](/resource/image/2017-10-19_211146.png)