### 概述
> 抽象工厂（Abstract Factory）模式，提供一个创建一系列相关或相互依赖对象的接口，而无须指定它们具体的类。抽象工厂模式又称为Kit模式，它是一种对象创建型模式。

### 产品等级与产品族
- 产品等级
产品等级即产品的继承结构，例如一个抽象类是电视机，其子类有海尔电视机、海信电视机、TCL电视机，则抽象电视机与具体品牌的电视机之间构成了一个产品等级结构，抽象电视机是父类，而具体品牌电视机则是子类。

- 产品族
产品族是指由同一个工厂生产的，位于不同产品等级结构中的一组，例如海尔电器厂生产的海尔电视机、海尔电冰箱，海尔电视机位于电视机产品等级结构中，而海尔电冰箱则位于电冰箱产品等级结构中，他们俩构成了一个产品族。

- 产品等级与产品族的示意图
    - ![产品等级与产品族的示意图](/resource/image/designPattern/产品等级与产品族的示意图.png)
    - 当系统所提供的工厂生产的具体产品并不是一个简单的对象，而是多个位于不同产品等级结构、属于不同类型的具体产品时，就可以使用抽象工厂模式。抽象工厂模式是所有形式的工厂模式中最为抽象和最具一般性的一种形式。

### 结构图
- ![抽象工厂](/resource/image/designPattern/abstractFactory.png)
- 4个角色
    - Abstract Factory (抽象工厂角色)：声明了一组用于创建一族产品的方法，每一个方法对应一种产品。
    - Concrete Factory (具体工厂角色)：实现了在抽象工厂中声明的创建产品的方法，生成一组具体产品，这些产品构成了一个产品族。
    - Abstract Product (抽象产品角色)：为每种产品声明接口，在抽象产品中声明了所有的业务方法。
    - Concrete Product (具体产品角色)：定义具体工厂生产的具体产品对象，实现在抽象产品接口中声明的业务方法。

### 主要优点
- 隔离了具体类的生成，使得客户并不需要知道什么被创建。因为这种隔离，因此更换一个具体工厂就变得相对容易。
- 当一个产品族中的多个对象被设计称一起工作时，它能够保证客户端始终只使用同一个产品族中的对象。
- 增加新的产品族很方便，无需修改已有系统，符合开闭原则。

### 主要缺点
- 增加新的产品等级结构很麻烦。因为需要对原有系统进行较大的修改，甚至需要修改抽象层代码，这必然会带来较大的不便，在这个角度，它违背了开闭（对扩展开放，对修改封闭）原则。

### 适用场景
- 用户无须关心对象的创建过程，需要将对象的创建和使用解耦 -> 这是所有工厂模式的使用前提
- 系统中有多余一个的产品族，而每次都只使用其中的某一种产品族。 -> 可以通过配置文件等方式来使得用户可以动态地改变产品族，也可以很方便地增加新的产品族
- 产品等级结构稳定！设计完成之后，不会向系统中增加新的产品等级结构或删除已有产品等级结构。 -> 并不太符合开闭原则

### 引用
- [周旭龙 - 设计模式的征途—4.抽象工厂（Abstract Factory）模式](https://www.cnblogs.com/edisonchou/p/6777901.html)