## BroadcastChannel
> 允许**同源**脚本跨 windows, tabs, frames, iframes 进行多对多通信

### 特点
- 同源传输
- 不可跨频道通信
- 不用持有其它iframes的句柄
- 多对多、全双工通讯
- 传输对象并不是深拷贝，而是引用地址

### API
- 参数
    - name 频道名称，用来区分不同频道（跨频道不可通信）
- 方法
    - postMessage 发送广播信息
    - onmessage 接受广播信息
    - close 关闭连接

### 通信过程
![BroadcastChannel](/resource/image/html/BroadcastChannel.png)

``` js
// page A
const a = new BroadcastChannel('app');
const list = ['A', 'B'];
a.postMessage({ list });
a.close();

// page B
const b = new BroadcastChannel('app')
b.onmessage = function (event) {
    event.data.list.push('C');
}
```

### 参考
- [MDN](https://developer.mozilla.org/en-US/docs/Web/API/Broadcast_Channel_API)
- [青青子衿](https://hexuanzhang.github.io/2018/03/02/BroadcastChannel/)

## window.postMessage

## SharedWorker

## MessageChannel

## 技术对比
| 类别 | 通讯方式 | 传输方式   | 传输对象 | 前提条件 | 优势 |
| :--  | --:     | :--    | :-- | :--  |:--  |:--  |
| BroadcastChannel  | 多对多 | 同源 | 对象引用地址 | | |
| window.postMessage  |  | 跨域 | 经过 structured clone algorithm 后的数据 | | |
| SharedWorker  |  |  |  | | 在服务器与客户端之间的状态管理、资源同步方面更具优势 |
| MessagChannel  | 一对一 |  |  | | |
| localStorage  |  | 同源 |  | | |

