## 严格模式（strict mode）
- 为 `JavaScript` 定义了一种不同的解析与执行模型

### 启用严格模式
``` js
// 在js文件顶部或函数内部的上方添加
"use strict";
```

## 关键字
- 可用于表示控制语句的开始或结束，或者用于执行特定操作等

## 保留字
- 有可能在将来被用作关键字

## `typeof` 操作符
- `undefined`
- `boolean`
- `string`
- `number`
- `object` 如果这个值是对象或 `null` (因为特殊值 `null` 被认为是一个空的对象引用)
- `function`

## `Undefined` 类型
> 使用 `var` 声明变量但未对其加以初始化时，这个变量的值就是 `undefined` 。`ECMA-262`第 3 版引入这个值是为了正式区分空对象指针( `null` )与未经初始化的变量。

``` js
var message;
alert(message == undefined); //true
```

### 未初始化的变量与未声明的变量
- 不同之处
    ``` js
    var message; // 这个变量声明之后默认取得了 undefined 值
    // 下面这个变量并没有声明
    // var age
    alert(message); // "undefined"
    alert(age); // 产生错误
    ```
- 相同之处
    ``` js
    var message; // 这个变量声明之后默认取得了 undefined 值
    // 下面这个变量并没有声明
    // var age
    alert(typeof message); // "undefined"
    alert(typeof age); // "undefined"
    ```

## `Null` 类型
-  `undefined` 值是派生自 `null` 值的
``` js
alert(null == undefined); //true
```

## `Boolean` 类型
- `True` 和 `False`（以及其他的混合大小写形式）都不是 `Boolean` 值
- 转型函数 `Boolean()`

## `Number` 类型
- 使用 `IEEE754` 格式来表示整数和浮点数值
- 十进制
- 八进制
    - 第一位必须是零（`0`），然后是八进制数字序列（`0～7`）
    - 如果字面值中的数值超出了范围，那么前导零将被忽略，后面的数值将被当作十进制数值解析
    ``` js
    var octalNum1 = 070; // 八进制的 56
    var octalNum2 = 079; // 无效的八进制数值——解析为 79
    var octalNum3 = 08; // 无效的八进制数值——解析为 8
    ```
    - 八进制字面量在严格模式下是无效的，会导致支持的 JavaScript 引擎抛出错误
- 十六进制
    - 前两位必须是 `0x`，后跟任何十六进制数字（`0～9` 及 `A～F`）。其中，字母 `A～F`
可以大写，也可以小写

### 浮点数值
- 保存浮点数值需要的内存空间是保存整数值的两倍
- 将那些小数点后面带有 `6个零` 以上的浮点数值转换为以 `e 表示法`（即科学计数法）表示的数值
- 浮点数值的最高精度是 `17位`小数，但在进行算术计算时其精确度远远不如整数