### 语法
- 双引号中的变量会被解析为变量的值
- 单引号中的变量不会被解析为变量的值，见示例`双、单引号中的变量`
- 双、单引号中的转义序列会被解析?
- 自增、自减运算符（见示例`自增、自减运算符`）

### 常用函数
- `is_string`    检测变量是否是字符串
- `trim` 去除一个字符串两端空格
- `rtrim` 是去除一个字符串右部空格
- `ltrim` 是去除一个字符串左部空格
- `strlen` 获取字符串的长度（字节数而非字符数）
- `mb_strlen` 获取字符串的长度（字节数而非字符数，可设置字符串的编码），见示例`获取字符串长度的几种方式`
- `substr`  返回字符串的子串
- `mb_substr`   返回字符串的子串，见示例`mb_substr函数注意`
- `str_replace` 子字符串替换 `mixed str_replace ( mixed $search , mixed $replace , mixed $subject [, int &$count ] )`，见示例`str_replace函数`
- `sprintf` 返回已格式化的字符串  `string sprintf ( string $format [, mixed $args [, mixed $... ]] )`，见示例`sprintf函数`
- `implode` 将一个一维数组的值转化为字符串 `string implode ( string $glue , array $pieces )`
- `explode` 使用一个字符串分割另一个字符串 `array explode ( string $delimiter , string $string [, int $limit ] )`

### 某些函数需要注意的地方
#### `strlen`/`substr`
- 当文件编码是`UTF-8`时
    - `strlen`：1个中文=3个字节
    - `substr`：1个中文=3个字节。所以截取时，参数中的长度须为3的整数倍。
#### `mb_strlen`/`mb_substr`
- `mb_strlen`
    - `utf8`：1个中文=1个字节
    - `gbk`：1个中文=2个字节，2个中文=3个字节。优先按照偶数个中文来计算。
    - `gb2312`：1个中文=2个字节。
- `mb_substr`
    - `utf8`：同`mb_strlen`
    - `gbk`：同`mb_strlen`
    - `gb2312`：同`mb_strlen`。当字符串包含中文、以中文开头，且中文大于2个时，截取参数长度最少为4的整数倍，否则会异常。见示例`mb_strlen函数注意`。

### 示例
#### 双、单引号中的变量
``` php
$t = "f";
echo 'hi,$t';
echo "hi,$t";
```
#### 获取字符串长度的几种方式
``` php
$str = "中文a字1符";
echo mb_detect_encoding($str).'<br>';//UTF-8 检测字符的编码
echo strlen($str).'<br>' ;//14
echo mb_strlen($str,'utf8').'<br>';//6 
echo mb_strlen($str,'gbk').'<br>';//8 
echo mb_strlen($str,'gb2312').'<br>';//10 
```
#### `mb_substr`函数注意
``` php
$str1 = "f字符";
$str2 = "中文字符";
echo mb_substr($str1,0,3,'gb2312'); //f字
echo mb_substr($str2,0,3,'gb2312'); //error
echo mb_substr($str2,0,4,'gb2312'); //中文
echo mb_substr($str2,0,5,'gb2312'); //error
echo mb_substr($str2,0,6,'gb2312'); //中文字
```
#### `str_replace`函数
``` php
//示例 1
$vowels = array("a", "e", "i", "o", "u", "A", "E", "I", "O", "U");
echo str_replace($vowels, "", "Hello World of PHP"); // Hll Wrld f PHP
//示例 2
$letters = array('a', 'p');
$fruit   = array('apple', 'pear');
$text    = 'a p';
$output  = str_replace($letters, $fruit, $text);
echo $output;	// 输出： apearpearle pear。 替换顺序： a p -> apple p -> apearpearle pear
```
#### `sprintf`函数
``` php
$num1 = 10.1;	// 注释：格式值 "%%" 返回百分号							
echo sprintf("%%b = %b",$num1); // 二进制数                 %b = 1010   
echo sprintf("%%d = %d",$num1); // 带符号的十进制数          %d = 10	
echo sprintf("%%e = %e",$num1); // 科学计数法（小写）        %e = 1.010000e+1		
echo sprintf("%%u = %u",$num1); // 不带符号的十进制数（正）   %u = 10			
echo sprintf("%%f = %.2f",$num1); // 浮点数（视本地设置）     %f = 10.10			
echo sprintf("%%F = %.2F",$num1); // 浮点数（不视本地设置）   %F = 10.10			
echo sprintf("%%o = %o",$num1); // 八进制数                 %o = 12	   
echo sprintf("%%s = %s",$num1); // 字符串                   %s = 10.1	   
echo sprintf("%%x = %x",$num1); // 十六进制数（小写）        %x = a	
echo sprintf("%%X = %X",$num1); // 十六进制数（大写）        %X = A	
```
#### 自增、自减运算符
``` php
$a = 'a';
$b = 'K9';
$c = '5';
$d = 'spaz';
$a++;   // string 'b' 
$b++;   // string 'L0' 
$c++;   // int 6
$d++;   // string 'spba' 
```