# 目录
- [目录](#目录)
  - [时间](#时间)
  - [字符串](#字符串)
    - [`_`匹配第N个字符](#_匹配第n个字符)
    - [连接字符串 concat(str1,str2,...)](#连接字符串-concatstr1str2)
    - [分组连接字符串 group\_concat( \[distinct\] 要连接的字段 \[order by 排序字段 asc/desc  \] \[separator '分隔符'\])](#分组连接字符串-group_concat-distinct-要连接的字段-order-by-排序字段-ascdesc---separator-分隔符)
    - [去除回车符，换行符，空格和水平制表符](#去除回车符换行符空格和水平制表符)
    - [TRIM - 从字符串中除去前导和尾随空格](#trim---从字符串中除去前导和尾随空格)
    - [FIND\_IN\_SET - 在逗号分隔的字符串列表中查找指定字符串的位置](#find_in_set---在逗号分隔的字符串列表中查找指定字符串的位置)
  - [查看系统变量](#查看系统变量)
  - [查看表大小](#查看表大小)
  - [查看表字段](#查看表字段)
  - [条件函数](#条件函数)
  - [查询缓存](#查询缓存)
  - [查询是否只读](#查询是否只读)
  - [查看隔离级别](#查看隔离级别)
  - [查看正在执行的线程](#查看正在执行的线程)

## 时间

``` sql
-- 格式化
-- %Y 年，4 位    %y 年，2 位    %m 月，数值(00-12)   %d 月的天，数值(00-31)
DATE_FORMAT(NOW(),'%Y-%m-%d')

-- 加减时间
DATE_ADD('2019-11-11 15:20:12',INTERVAL -1 DAY)
-- 返回：2019-11-10 15:20:12，类型为CHAR(29)

-- 时间差值
TIMESTAMPDIFF(SECOND,'2020-11-20 14:00:01','2020-11-20 14:00:02');
-- 返回：1 ，类型为 BIGINT(21)
```

## 字符串

### `_`匹配第N个字符

``` sql
SELECT name FROM world WHERE name LIKE '_t%'
-- 返回：Italy
```

### 连接字符串 concat(str1,str2,...)

``` sql
SELECT CONCAT('2019-11-07 15:20:',14.000001);
-- 返回：2019-11-07 15:20:14.000001，类型为VARCHAR(27)
```

### 分组连接字符串 group_concat( [distinct] 要连接的字段 [order by 排序字段 asc/desc  ] [separator '分隔符'])

``` sql
SELECT GROUP_CONCAT(CONCAT(AlarmTime,'%%%',AlarmContent) SEPARATOR '###')
FROM device_alarm
WHERE AlarmTime >= DATE_ADD('2019-11-07 15:20:12',INTERVAL -1 DAY)
AND AlarmTime <='2019-11-07 15:20:12'
GROUP BY DeviceId
HAVING COUNT(1) >= 3;
-- 返回：2019-11-07 03:15:28%%%漏电跳闸###2019-11-07 02:27:28%%%漏电跳闸###……，类型为VARCHAR(341)
```

### 去除回车符，换行符，空格和水平制表符

- CHAR(9)=>水平制表符
- CHAR(10)=>换行符
- CHAR(13)=>回车符

``` sql
UPDATE table SET fund_name = REPLACE(REPLACE(REPLACE(REPLACE(fund_name,CHAR(13),''),CHAR(10),''),CHAR(9),''),' ','')
```

### TRIM - 从字符串中除去前导和尾随空格

``` sql
SELECT TRIM(' MySQL TRIM Function ');
-- MySQL TRIM Function
```

### FIND_IN_SET - 在逗号分隔的字符串列表中查找指定字符串的位置

``` sql
SELECT FIND_IN_SET(4,'18,4,22,24, 35, 61')  -- 2
SELECT FIND_IN_SET(4,'18,14,22,24, 35, 61')  -- 0
```

## 查看系统变量

``` sql
show variables like '%group_concat%';
-- 返回：group_concat_max_len	1024

-- 方法一：修改MySQL配置文件my.cnf，在[mysqld]节点中添加 group_concat_max_len = 10240
-- 方法二：在会话中实现，全局或当前session中
SET GLOBAL group_concat_max_len=10240;
SET SESSION group_concat_max_len=10240;
```

## 查看表大小

``` sql
select t.TABLE_NAME AS name,t.DATA_LENGTH / 1024 / 1024 AS `size(MB)`,t.TABLE_ROWS as rowCount from information_schema.TABLES t where t.TABLE_SCHEMA = 'hdkj' and t.TABLE_NAME like 'performance%' order by t.DATA_LENGTH desc
```

## 查看表字段

``` sql
SELECT
  c.TABLE_NAME AS tablName,
  c.ORDINAL_POSITION AS number,
  c.COLUMN_NAME AS `column`,
  c.DATA_TYPE AS datatype,
  CASE c.COLUMN_KEY WHEN 'PRI' THEN '1' ELSE '' END AS `key`,
  CASE c.IS_NULLABLE WHEN 'YES' THEN '1' ELSE '' END AS isnullable,
  c.COLUMN_COMMENT AS remark
FROM information_schema.COLUMNS c
WHERE c.TABLE_SCHEMA = 'hdkj'
AND c.COLUMN_NAME LIKE '%Organ%';
```

## 条件函数

``` sql
SELECT IF(TRUE,'A','B');    -- 输出结果：A
SELECT IF(FALSE,'A','B');   -- 输出结果：B

SELECT IFNULL('A','B'); -- 输出结果：A
SELECT IFNULL(NULL,'B');    -- 输出结果：B

SELECT NULLIF('A','A');     -- 输出结果：null
SELECT NULLIF('A','B');     -- 输出结果：A

SELECT ISNULL(NULL);        -- 输出结果：1
SELECT ISNULL('A');     -- 输出结果：0

SELECT IF(ISNULL(NULL),'A','B');    -- 输出结果：A
SELECT IF(ISNULL('C'),'A','B');     -- 输出结果：B
```

## 查询缓存

- 去除缓存
``` sql
select SQL_NO_CACHE * from table
```

## 查询是否只读

``` sql
select @@read_only;
```

## 查看隔离级别

``` sql
select @@tx_isolation （mysql版本 8.0 之前）
select @@transaction_isolation （mysql版本 8.0 以后）
```

## 查看正在执行的线程

``` sql
select  ID,HOST,DB,COMMAND,`TIME`,STATE,INFO from information_schema.processlist WHERE HOST LIKE '192.168.1.171:%'
```