# 目录

- [目录](#目录)
  - [命令选项](#命令选项)
  - [镜像](#镜像)
  - [搜索镜像](#搜索镜像)
    - [获取镜像](#获取镜像)
    - [列出镜像](#列出镜像)
  - [容器](#容器)
    - [启动容器](#启动容器)
    - [列出容器](#列出容器)
    - [终止容器](#终止容器)
    - [删除单个容器](#删除单个容器)
    - [进入容器](#进入容器)
  - [数据卷](#数据卷)
    - [查看所有的数据卷](#查看所有的数据卷)
    - [创建一个数据卷](#创建一个数据卷)
    - [查看指定数据卷](#查看指定数据卷)
    - [启动一个容器时挂载数据卷](#启动一个容器时挂载数据卷)
    - [删除数据卷](#删除数据卷)
  - [挂载主机目录](#挂载主机目录)
    - [挂载一个主机目录作为数据卷](#挂载一个主机目录作为数据卷)
    - [修改挂载目录权限](#修改挂载目录权限)

## 命令选项

- `-t` 让Docker分配一个伪终端（pseudo-tty）并绑定到容器的标准输入上
- `-i` 让容器的标准输入保持打开
- `-d` 容器会在后台运行

## 镜像

## 搜索镜像

``` cmd
docker search key_word
```

### 获取镜像

``` cmd
docker pull [选项] [Docker Registry 地址[:端口号]/]仓库名[:标签]
```

### 列出镜像

``` cmd
docker images
```

## 容器

### 启动容器

- 利用镜像创建并启动
  ``` cmd
  docker run -t -i centos:latest /bin/bash
  ```
- 启动已终止容器
  ``` cmd
  docker start -i container_id
  ```

### 列出容器

- 当前所有正在运行的
  ``` cmd
  docker ps
  ```
- 所有的（包括未运行的）
  ``` cmd
  docker ps -a
  ```

### 终止容器

- 手动执行命令终止
  ``` cmd
  docker stop container_id
  ```
- 自动终止
  - 当 Docker 容器中指定的应用终结时，容器也自动终止
- 终止状态的容器可以用 `docker container ls -a` 命令

### 删除单个容器
  ``` cmd
  docker rm Name/ID
  ```

### 进入容器

``` cmd
docker exec -it container_name bash
docker exec -it --user root container_name bash
```

## 数据卷

### 查看所有的数据卷

``` cmd
docker volume ls
```

### 创建一个数据卷

``` cmd
docker volume create name
```

### 查看指定数据卷

``` cmd
docker volume inspect centos-vol
```

### 启动一个容器时挂载数据卷

``` cmd
docker run -t -i --mount source=centos-vol,target=/webapp centos:latest /bin/bash 
```

### 删除数据卷

``` cmd
docker volume rm centos-vol
```

## 挂载主机目录

### 挂载一个主机目录作为数据卷

``` cmd
docker run -t -i --mount type=bind,source=/D/docker-volume/centos-vol,target=/opt/webapp centos:latest /bin/bash 
```

### 修改挂载目录权限

``` cmd
docker run -t -i --mount type=bind,source=/D/docker-volume/centos-vol,target=/opt/webapp,readonly centos:latest /bin/bash 
```
