## 参考资料

- [Java技术——你真的了解String类的intern()方法吗](https://blog.csdn.net/seu_calvin/article/details/52291082)
- [面试别再问我String了](https://mp.weixin.qq.com/s?__biz=MzAwNTM0ODY1Mg==&mid=2457116104&idx=1&sn=e6e81aeb793056cc0f0ba61b3daeef37&chksm=8c9e318bbbe9b89dc9e5969c3991926921deb6e5d4fe1ce667149d11bf5accfbd4191d1ee09f&token=325711551&lang=zh_CN#rd)
- [java基础：String — 字符串常量池与intern(二）](https://juejin.im/post/5c160420518825235a05301e)

> 在jdk6中，常量池的位置在永久代（方法区）中，此时常量池中存储的是**对象**。在jdk7中，常量池的位置在堆中，此时，常量池存储的就是**引用**了。在jdk8中，永久代（方法区）被元空间取代了。

## 使用字面量

- 去常量池中查找是否存在相同内容的对象
  - 当存在时直接返回对象保存的地址
  - 当不存在时，会先在堆中创建`String Object`对象，再在字符串常量池中创建一个引用对象（此时引用对象保存的是`String Object`对象的地址），并返回此地址

## 调用 strA.intern()

- 去常量池中查找是否存在相同内容的对象
  - 当存在时直接返回对象保存的地址
  - 当不存在时，在字符串常量池中创建一个引用对象（此时引用对象保存的是`strA`的地址），并返回`strA`的地址

## String

``` java

// 以下例子在jdk1.8测试

String str1 = "HelloFlyapi";
String str2 = "HelloFlyapi";
// 在堆内存创建对象A，再去常量池中查找是否存在"HelloFlyapi"
String str3 = new String("HelloFlyapi");
String str4 = "Hello";
String str5 = "Flyapi";
// 由于”Hello”和”Flyapi”都是常量，编译时，会被自动编译为 String str6 = “HelloFlyapi”
String str6 = "Hello" + "Flyapi";
// JVM会在堆（heap）中创建一个以str4为基础的一个StringBuilder对象，然后调用StringBuilder的append()方法完成与str5的合并，之后会调用toString()方法在堆（heap）中创建一个String对象，并把这个String对象的引用赋给str7。
String str7 = str4 + str5;

// true
System.out.println("str1 == str2 result: " + (str1 == str2));
// false
System.out.println("str1 == str3 result: " + (str1 == str3));
// true
System.out.println("str1 == str6 result: " + (str1 == str6));
// false
System.out.println("str1 == str7 result: " + (str1 == str7));
// true
System.out.println("str1 == str7.intern() result: " + (str1 == str7.intern()));
// false
System.out.println("str3 == str3.intern() result: " + (str3 == str3.intern()));

String str22 = "SEUCalvin";
String str11 = new String("SEU") + new String("Calvin");
String copyStr11 = str11.intern();
// false
System.out.println(copyStr11 == str11);
// false
System.out.println(str11 == "SEUCalvin");
```
