## 基本类型
- 整型
    - `byte` (1个字节)
    - `short`(2个字节)
    - `int`  (4个字节)
    - `long` (8个字节)
    - `BigInteger`(8+个字节)(非基本类型)
- 浮点
    - `float` (4个字节)
    - `double`(8个字节)
    - `BigDecimal`(非基本类型)
- 字符
    - `char`
    - `string`(非基本类型)
- 布尔
    - `boolean`
## 基本类型和对应的包装类
|基本类型|包装类|
|:---:|:---:|
|boolean|Boolean|
|byte|Byte|
|short|Short|
|int|Integer|
|long|Long|
|float|Float|
|double|Double|
|char|Character|

## 整型
### `int`
- `Integer.MIN_VALUE` 最小整数，为`0x80000000`
- `Integer.MAX_VALUE` 最大整数，为`0x7fffffff`

### `long`
- 带 `L` 后缀

## 浮点
- 可用十六进制表示 
``` java
float f = 0.0009765625F;
boolean check = f == 0x1.0p-10;// check equals true.
```
### `float`
- 带 `F` 后缀
### `double`
- 带 `D` 后缀或不带
- `Double.NEGATIVE_INFINITY` 负无穷，为`-1.0 / 0.0`
- `Double.POSITIVE_INFINITY` 正无穷，为`1.0 / 0.0`
- `Double.NaN` 非数值，为`0.0d / 0.0`
- 常用方法
    - `Double.isNaN(x)` 检查 `x` 是否为 `NaN`
    - `Double.isInfinite(x)` 检查 `x` 是否为正负无穷
    - `Double.isFinite(x)` 检查 `x` 既不是正负无穷，也不是 `NaN`

## 数字类型转换
下面的转换是合法的，但损失了信息：
- `int` => `float`
    ``` java
    float f = 123456789;
    boolean check = f==1.234567892E8;// check equals true.
    ```
- `long` => `float` 或 `double`

## `String`
- 在 Java 虚拟机中，每个文字串只有一个实例
- `==` 比较对象引用
- `equals` 比较值

## 进制
- 二进制 `0b` 后缀。示例：0b1001(二进制) = 9(十进制)
- 八进制 `0` 后缀。示例：011(八进制) = 9(十进制)
- 十进制 
- 十六进制 `0x` 后缀

## 常用方法
- `Integer.compare(int x, int y)`
- `Double.compare(double d1, double d2)` 能计算NaN