## 泛型类
- 类型参数不能用基本类型。例如，`Entry<string,int>`

## 泛型方法
``` java
public class Arrays {
    public static <T> void swap(T[] array, int i, int j) { ... }
}
...
String [] friends = { "Tom","Jim" };
Arrays.swap(friends,0,1);         // 隐式指定类型
Arrays.<String>swap(friends,0,1); // 显式指定类型
```

## `extends` 类型限定
``` java
public static <T extends AutoCloseable> void closeAll(ArrayList<T> elems) throws Exception {
    for (T elem : elems) elem.close();
}
```
- 类排在第一个位置
- 可有多个接口

## 类型的协变性
- 父类数组作实参时，子类数组可作为形参
    ``` java
    public static void printAll(Employee[] staff, Predicate<? super Employee> filter) { ... }
    ...
    Predicate<? super Employee> filter = x -> x.getSalary() > 6000;
    Employees.printAll(new Employee[]{
        new Employee("Tom", 6000),
        new Employee("Jim", 8000)
    }, filter);
    Employees.printAll(new Manager[]{
        new Manager("Tom2", 6000),
        new Manager("Jim2", 8000)
    }, filter);
    ```
- `extends` 子类型通配符
    - `? extends T`
- `super` 父类型通配符
    - `? super T`
- PECS
    - Producter(生产者) -> `extends`
    - Consumer(消费者) -> `Super`

## 类型擦除
> Java泛型的处理几乎都在编译器中进行，编译器生成的bytecode是不包含泛型信息的，泛型类型信息将在编译处理是被擦除，这个过程即类型擦除。
- 泛型的 `class` 对象是相同的
- 泛型数组初始化时不能声明泛型类型
- `instanceof` 不允许后接泛型参数

## 桥方法
- 桥方法的机制来避免类型擦除与多态发生冲突。桥方法并不需要自己手动生成，是编译器自动完成的。
