## 接口
> 接口是面向对象编程的一个关键特性：它们让你在不提供实现的情况下，指定应该做什么。

- 声明接口
- 实现接口 `implements`
    ``` java
    public final class LocalDateTime implements Temporal, TemporalAdjuster, ChronoLocalDateTime<LocalDate>, Serializable { ... }
    ```
- 测试对象是否为期望的类型 `object instanceof Type`
    ``` java
    if ( sequence instanceof DigitSequence ) { ... }
    ```
- 继承接口 `extend`
    ``` java
    public interface Channel extends Closeable { ... }
    ```
- 静态方法
- 默认方法
    > 用途：接口演化。接口后续的维护中，如果添加了新方法，以前继承该接口的类将无法编译。
    - 方法必须添加 `default` 修饰符
    - 多个接口中的默认方法冲突
        ``` java
        public interface A {
            default int getId() { return 0; }
        }
        public interface B {
            default int getId() { return 1; }
        }
        public class C {
            public int getId(){
                return B.super.getId(); // 注意 super 关键字
            }
        }
        ```