## 方法覆盖
- `super` 不是对象的引用，而是绕过动态查找方法并调用特定方法的指令。
``` java
public class Employee {
    private String name;
    private double salary;

    public double getSalary() {
        return salary;
    }
}
public class Manager extends Employee {
    private double bonus;

    public void setBonus(double bonus) {
        this.bonus = bonus;
    }

    // Overrides superclass method
    public double getSalary() {
        return super.getSalary() + bonus;
    }
}
```
- 被覆盖方法的访问控制级别可以不一样
    > 例如父类的showMess方法的访问级别为protected的，而子类覆盖的showMess方法访问级别则为public的。但子类的访问级别必须要高于父类被覆盖方法的访问级别，如果父类是public的而子类是protected的则是错误的。
- 父类方法被定义为`private`、`static`、`final`的则不能被覆盖
- 方法的返回类型可以不一样
    > 重写的方法名称，参数及类型必须一致，但没有提返回类型。这里，返回类型可以不一样，但类型有限制。子类的返回类型可以是更具体的对象，例如可以将Employee类的返回类型改为Object也正确。而倒过来则错误。
- 在方法调用时先会在子类中找覆盖的方法，如果子类中没有则会在父类中去找
- `@Override` 
    - 增加代码的可读性，看到标签就知道这是从父类重写的方法
    - IDE会根据这个标签来检查你的代码

## 转换
- `instanceof`
``` java
Employee empl = new Mananger();
if(empl instanceof Manager){
    ...
}
```

## 枚举
- 定义
    ``` java
    public enum Size { SMALL,MEDIUM,LARGE };
    ...
    Size small = Size.valueOf("SMALL");
    Size[] allValues = Size.values();
    ```
- 可包含构造函数（总被 `private` 修饰）、方法、域
- 单个枚举实例可以添加方法。添加的方法也可以覆盖枚举类中定义的方法
- 可包含静态成员。枚举中的常量在静态成员之前构建。
