1. 在 `pom.xml` 中添加 `spring-boot-starter-web`、`spring-boot-starter-thymeleaf` 的依赖
``` xml
<dependency>
    <groupId>org.springframework.boot</groupId>
    <artifactId>spring-boot-starter-web</artifactId>
</dependency>
<dependency>
    <groupId>org.springframework.boot</groupId>
    <artifactId>spring-boot-starter-thymeleaf</artifactId>
</dependency>
```
2. 在 `application.properties` 中配置数据库连接
``` xml
# 是否支持批量上传(默认值 true)
spring.servlet.multipart.enabled=true
# 上传文件的临时目录
spring.servlet.multipart.location=
# 上传文件最大为 1M
spring.servlet.multipart.max-file-size=1048576
# 上传请求最大为 10M
spring.servlet.multipart.max-request-size=10485760
# 文件大小阈值，当大于这个阈值时将写入到磁盘，否则存在内存中（默认值0）
spring.servlet.multipart.file-size-threshold=0
# 判断是否要延迟解析文件（相当于懒加载，默认为 false）
spring.servlet.multipart.resolve-lazily=false
```
3. 新建 `/resources/static/FileUpload/index.html`
``` html
<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <meta charset="UTF-8">
    <title>文件上传</title>
</head>
<body>
    <h2>单一文件上传示例</h2>
    <div>
        <form method="POST" enctype="multipart/form-data" action="/dev/uploads/upload1">
            <p>
                文件1：<input type="file" name="file"/>
                <input type="submit" value="上传"/>
            </p>
        </form>
    </div>
</body>
</html>
```
4. 新建 `FileUploadController.java`
``` java
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping("/uploads")
public class FileUploadController {

    private static final Logger log = LoggerFactory.getLogger(FileUploadController.class);

    @GetMapping
    public ModelAndView index() {
        ModelAndView view = new ModelAndView();
        view.setViewName("/FileUpload/index");
        return view;
    }


    @PostMapping("/upload1")
    @ResponseBody
    public Map<String, String> upload1(@RequestParam("file") MultipartFile file) throws IOException {
        log.info("[文件类型] - [{}]", file.getContentType());
        log.info("[文件名称] - [{}]", file.getOriginalFilename());
        log.info("[文件大小] - [{}]", file.getSize());
        // TODO 将文件写入到指定目录（具体开发中有可能是将文件写入到云存储/或者指定目录通过 Nginx 进行 gzip 压缩和反向代理，此处只是为了演示故将地址写成本地电脑指定目录）
        file.transferTo(new File("C:\\" + file.getOriginalFilename()));
        Map<String, String> result = new HashMap<>(16);
        result.put("contentType", file.getContentType());
        result.put("fileName", file.getOriginalFilename());
        result.put("fileSize", file.getSize() + "");
        return result;
    }
}
```
5. 使用浏览器访问 `http://localhost:9090/dev/uploads`，选择文件上传，可在 `IntelliJ IDEA` 中 `Console` 窗口看到如下输出
``` txt
2018-08-06 20:30:36.680  INFO 15564 --- [nio-9090-exec-7] o.n.d.controller.FileUploadController    : [文件类型] - [image/png]
2018-08-06 20:30:37.333  INFO 15564 --- [nio-9090-exec-7] o.n.d.controller.FileUploadController    : [文件名称] - [2018-08-05_094034.png]
2018-08-06 20:30:38.086  INFO 15564 --- [nio-9090-exec-7] o.n.d.controller.FileUploadController    : [文件大小] - [66922]
```